Package: GenericML
Title: Generic Machine Learning Inference
Version: 0.2.2
Authors@R: 
    c(person(given = "Max",
           family = "Welz",
           role = c("aut", "cre"),
           email = "welz@ese.eur.nl",
           comment = c(ORCID = "0000-0003-2945-1860")),
      person(given = "Andreas",
             family = "Alfons",
             role = "aut",
             email = "alfons@ese.eur.nl",
             comment = c(ORCID = "0000-0002-2513-3788")), 
      person(given = "Mert",
             family = "Demirer",
             role = "aut",
             email = "mdemirer@mit.edu"), 
      person(given = "Victor",
             family = "Chernozhukov",
             role = "aut",
             email = "vchern@mit.edu")) 
Author: Max Welz [aut, cre] (<https://orcid.org/0000-0003-2945-1860>),
  Andreas Alfons [aut] (<https://orcid.org/0000-0002-2513-3788>),
  Mert Demirer [aut],
  Victor Chernozhukov [aut]
Maintainer: Max Welz <welz@ese.eur.nl>
Description: Generic Machine Learning Inference on heterogeneous treatment effects in randomized experiments as proposed in Chernozhukov, Demirer, Duflo and Fernández-Val (2020) <arXiv:1712.04802>. This package's workhorse is the 'mlr3' framework of Lang et al. (2019) <doi:10.21105/joss.01903>, which enables the specification of a wide variety of machine learners. The main functionality, GenericML(), runs Algorithm 1 in Chernozhukov, Demirer, Duflo and Fernández-Val (2020) <arXiv:1712.04802> for a suite of user-specified machine learners. All steps in the algorithm are customizable via setup functions. Methods for printing and plotting are available for objects returned by GenericML(). Parallel computing is supported.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.2.0
URL: https://github.com/mwelz/GenericML/
BugReports: https://github.com/mwelz/GenericML/issues/
Depends: ggplot2, mlr3, mlr3learners
Imports: sandwich, lmtest, splitstackshape, stats, parallel, abind
Suggests: glmnet, ranger, rpart, e1071, xgboost, kknn, DiceKriging,
        testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2022-06-18 07:33:48 UTC; mwelz
Repository: CRAN
Date/Publication: 2022-06-18 08:00:09 UTC
Built: R 4.6.0; ; 2026-04-26 13:10:46 UTC; unix
