Package: GenericML
Title: Generic Machine Learning Inference
Version: 0.2.3
Authors@R: 
    c(person(given = "Max",
           family = "Welz",
           role = c("aut", "cre"),
           email = "max.welz@uzh.ch",
           comment = c(ORCID = "0000-0003-2945-1860")),
      person(given = "Andreas",
             family = "Alfons",
             role = "aut",
             email = "alfons@ese.eur.nl",
             comment = c(ORCID = "0000-0002-2513-3788")), 
      person(given = "Mert",
             family = "Demirer",
             role = "aut",
             email = "mdemirer@mit.edu"), 
      person(given = "Victor",
             family = "Chernozhukov",
             role = "aut",
             email = "vchern@mit.edu")) 
Author: Max Welz [aut, cre] (ORCID: <https://orcid.org/0000-0003-2945-1860>),
  Andreas Alfons [aut] (ORCID: <https://orcid.org/0000-0002-2513-3788>),
  Mert Demirer [aut],
  Victor Chernozhukov [aut]
Maintainer: Max Welz <max.welz@uzh.ch>
Description: Generic Machine Learning Inference on heterogeneous treatment effects in randomized experiments as proposed in Chernozhukov, Demirer, Duflo and Fernández-Val (2020) <doi:10.48550/arXiv.1712.04802>. This package's workhorse is the 'mlr3' framework of Lang et al. (2019) <doi:10.21105/joss.01903>, which enables the specification of a wide variety of machine learners. The main functionality, GenericML(), runs Algorithm 1 in Chernozhukov, Demirer, Duflo and Fernández-Val (2020) <doi:10.48550/arXiv.1712.04802> for a suite of user-specified machine learners. All steps in the algorithm are customizable via setup functions. Methods for printing and plotting are available for objects returned by GenericML(). Parallel computing is supported.
License: GPL (>= 3)
Encoding: UTF-8
URL: https://github.com/mwelz/GenericML/
BugReports: https://github.com/mwelz/GenericML/issues/
Depends: ggplot2, mlr3, mlr3learners
Imports: sandwich, lmtest, splitstackshape, stats, parallel, abind,
        weights
Suggests: glmnet, ranger, rpart, e1071, xgboost, kknn, DiceKriging,
        MASS, nnet, testthat (>= 3.0.0)
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-02 07:56:39 UTC; mwelz
Repository: CRAN
Date/Publication: 2026-07-03 16:00:10 UTC
Built: R 4.5.2; ; 2026-07-03 19:22:15 UTC; unix
