# Generated by roxygen2: do not edit by hand

S3method(plot,BLP_info)
S3method(plot,CLAN_info)
S3method(plot,GATES_info)
S3method(plot,GenericML)
S3method(print,BLP_info)
S3method(print,CLAN_info)
S3method(print,GATES_info)
S3method(print,GenericML)
S3method(print,best)
S3method(print,heterogeneity_CLAN)
export(BLP)
export(CLAN)
export(GATES)
export(GenericML)
export(GenericML_combine)
export(GenericML_single)
export(Med)
export(TrueIfUnix)
export(get_BLP)
export(get_CLAN)
export(get_GATES)
export(get_best)
export(heterogeneity_CLAN)
export(lambda_parameters)
export(propensity_score)
export(proxy_BCA)
export(proxy_CATE)
export(quantile_group)
export(setup_X1)
export(setup_diff)
export(setup_plot)
export(setup_stratify)
export(setup_vcov)
import(ggplot2)
import(mlr3)
import(mlr3learners)
import(sandwich)
